/*
 * Sakreble! - open word game.
 * Copyright (C) 2004 Marcin Gajda
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "alphabet.h"
#include "dupa.h"

Alphabet::Alphabet()
{
	no_polish_letters_visible = false; // Tzn. zakladam, ze uzytkownik ma dobrze skonfigurowanego XSerwera

	for( int i=0; i<256; i++ )
		letterPoints[i] = 0;

	letterPoints[ (unsigned char)'a' ] = 1;
	letterPoints[ (unsigned char)'b' ] = 3;
	letterPoints[ (unsigned char)'c' ] = 2;
	letterPoints[ (unsigned char)'d' ] = 2;
	letterPoints[ (unsigned char)'e' ] = 1;
	letterPoints[ (unsigned char)'f' ] = 4;
	letterPoints[ (unsigned char)'g' ] = 3;
	letterPoints[ (unsigned char)'h' ] = 3;
	letterPoints[ (unsigned char)'i' ] = 1;
	letterPoints[ (unsigned char)'j' ] = 3;
	letterPoints[ (unsigned char)'k' ] = 2;
	letterPoints[ (unsigned char)'l' ] = 2;
	letterPoints[ (unsigned char)'m' ] = 2;
	letterPoints[ (unsigned char)'n' ] = 1;
	letterPoints[ (unsigned char)'o' ] = 1;
	letterPoints[ (unsigned char)'p' ] = 2;
	letterPoints[ (unsigned char)'r' ] = 1;
	letterPoints[ (unsigned char)'s' ] = 1;
	letterPoints[ (unsigned char)'t' ] = 2;
	letterPoints[ (unsigned char)'u' ] = 3;
	letterPoints[ (unsigned char)'w' ] = 1;
	letterPoints[ (unsigned char)'y' ] = 2;
	letterPoints[ (unsigned char)'z' ] = 1;
	letterPoints[ (unsigned char)'' ] = 5;
	letterPoints[ (unsigned char)'' ] = 6;
	letterPoints[ (unsigned char)'' ] = 5;
	letterPoints[ (unsigned char)'' ] = 5;
	letterPoints[ (unsigned char)'' ] = 5;
	letterPoints[ (unsigned char)'' ] = 7;
	letterPoints[ (unsigned char)'' ] = 7;
	letterPoints[ (unsigned char)'' ] = 3;
	letterPoints[ (unsigned char)'' ] = 5;
	letterPoints[ (unsigned char)BLANK ] = 0;

	alphabetLetters[0] = 'a';
	alphabetLetters[1] = '';
	alphabetLetters[2] = 'b';
	alphabetLetters[3] = 'c';
	alphabetLetters[4] = '';
	alphabetLetters[5] = 'd';
	alphabetLetters[6] = 'e';
	alphabetLetters[7] = '';
	alphabetLetters[8] = 'f';
	alphabetLetters[9] = 'g';
	alphabetLetters[10] = 'h';
	alphabetLetters[11] = 'i';
	alphabetLetters[12] = 'j';
	alphabetLetters[13] = 'k';
	alphabetLetters[14] = 'l';
	alphabetLetters[15] = '';
	alphabetLetters[16] = 'm';
	alphabetLetters[17] = 'n';
	alphabetLetters[18] = '';
	alphabetLetters[19] = 'o';
	alphabetLetters[20] = '';
	alphabetLetters[21] = 'p';
	alphabetLetters[22] = 'r';
	alphabetLetters[23] = 's';
	alphabetLetters[24] = '';
	alphabetLetters[25] = 't';
	alphabetLetters[26] = 'u';
	alphabetLetters[27] = 'w';
	alphabetLetters[28] = 'y';
	alphabetLetters[29] = 'z';
	alphabetLetters[30] = '';
	alphabetLetters[31] = '';
	alphabetLetters[32] = BLANK;

	initializeLetters();
}

/*
 * Tej funkcji uzywaja stand i gameboard, tak wiec trzeba uzaleznic od czy_polskie
 */
char Alphabet::upperCaseDependOnPolishLettersVisibility( char c )
{
	if( c >= 'a' && c <= 'z' )
		return c - 32;

	if( no_polish_letters_visible )
		switch( c )
		{
			/*
			 * Trzeba tak, aby wyswietlaly sie blanks zastapione na polska litere
			 */
			case '':
			case '': return 'A';

			case '':
			case '': return 'E';

			case '':
			case '': return 'O';

			case '':
			case '': return 'S';

			case '':
			case '': return 'C';

			case '':
			case '': return 'L';

			case '':
			case '': return 'Z';

			case '':
			case '': return 'Z';

			case '':
			case '': return 'N';

			case BLANK: return ' ';
			default : return c;
		}
	else
		switch( c )
		{
			case '': return '';
			case '': return '';
			case '': return '';
			case '': return '';
			case '': return '';
			case '': return '';
			case '': return '';
			case '': return '';
			case '': return '';
			case BLANK: return ' ';
			default : return c;
		}
}

/*
 * Ta funkcja uzywana jest przez modul wyszukiwania the_bestch slow, tak wiec
 * musi byc niewraziwa na czy_polskie
 */
char Alphabet::upperCase( char c )
{
	if( c >= 'a' && c <= 'z' )
		return c - 32;

	switch( c )
	{
		case '': return '';
		case '': return '';
		case '': return '';
		case '': return '';
		case '': return '';
		case '': return '';
		case '': return '';
		case '': return '';
		case '': return '';
		default : return c;
	}
}

char Alphabet::lowerCase( char c )
{
	if( c >= 'A' && c <= 'Z' )
		return c + 32;

	switch( c )
	{
		case '': return '';
		case '': return '';
		case '': return '';
		case '': return '';
		case '': return '';
		case '': return '';
		case '': return '';
		case '': return '';
		case '': return '';

		default : return c;
	}
}

void Alphabet::initializeLetters()
{
	for( int i=0; i<256; i++ )
		letters[i] = 0;

	letters[ (unsigned char)'a' ] = 8;
	letters[ (unsigned char)'b' ] = 2;
	letters[ (unsigned char)'c' ] = 3;
	letters[ (unsigned char)'d' ] = 3;
	letters[ (unsigned char)'e' ] = 7;
	letters[ (unsigned char)'f' ] = 2;
	letters[ (unsigned char)'g' ] = 2;
	letters[ (unsigned char)'h' ] = 2;
	letters[ (unsigned char)'i' ] = 8;
	letters[ (unsigned char)'j' ] = 2;
	letters[ (unsigned char)'k' ] = 3;
	letters[ (unsigned char)'l' ] = 3;
	letters[ (unsigned char)'m' ] = 3;
	letters[ (unsigned char)'n' ] = 5;
	letters[ (unsigned char)'o' ] = 6;
	letters[ (unsigned char)'p' ] = 3;
	letters[ (unsigned char)'r' ] = 4;
	letters[ (unsigned char)'s' ] = 4;
	letters[ (unsigned char)'t' ] = 3;
	letters[ (unsigned char)'u' ] = 2;
	letters[ (unsigned char)'w' ] = 4;
	letters[ (unsigned char)'y' ] = 4;
	letters[ (unsigned char)'z' ] = 5;
	letters[ (unsigned char)'' ] = 1;
	letters[ (unsigned char)'' ] = 1;
	letters[ (unsigned char)'' ] = 1;
	letters[ (unsigned char)'' ] = 1;
	letters[ (unsigned char)'' ] = 1;
	letters[ (unsigned char)'' ] = 1;
	letters[ (unsigned char)'' ] = 1;
	letters[ (unsigned char)'' ] = 2;
	letters[ (unsigned char)'' ] = 1;
	letters[ (unsigned char)BLANK ] = 2;

	lettersLeft = 0;
	for( int i=0; i<256; i++ )
		lettersLeft += letters[i];
}

int Alphabet::isUpperCase( char c )
{
	if( c >= 'A' && c <= 'Z' )
		return 1;

	switch( c )
	{
		case '':
		case '':
		case '':
		case '':
		case '':
		case '':
		case '':
		case '':
		case '':
			return 1;

		default:
			return 0;
	}
}

char Alphabet::addOgonek( char c )
{
	switch( c )
	{
		case '':
		case '': return '`';
		case '':
		case '': return '`';
		case '':
		case '': return 0x27; // tzn. znaczek '
		case '':
		case '': return 0x27;
		case '':
		case '': return 0x27;
		case '':
		case '': return '/';
		case '':
		case '': return '.';
		case '':
		case '': return 0x27;
		case '':
		case '': return 0x27;
		default : return 0;
	}
}

int Alphabet::whereHorizontalOgonek( char c )
{
	switch( c )
	{
		case '':
		case '': return 16;
		case '':
		case '': return 16;
		case '':
		case '': return 14;
		case '':
		case '': return 14;
		case '':
		case '': return 14;
		case '':
		case '': return 13;
		case '':
		case '': return 12;
		case '':
		case '': return 13;
		case '':
		case '': return 14;
		default : return 0;
	}
}

int Alphabet::whereVerticalOgonek( char c )
{
	switch( c )
	{
		case '':
		case '': return 34;
		case '':
		case '': return 34;
		case '':
		case '': return 16;
		case '':
		case '': return 16;
		case '':
		case '': return 16;
		case '':
		case '': return 22;
		case '':
		case '': return 8;
		case '':
		case '': return 16;
		case '':
		case '': return 16;
		default : return c;
	}
}

int Alphabet::isOgonekNeeded( char c )
{
	if( ( c >= 'a' && c <= 'z' ) || ( c >= 'A' && c <= 'Z' ) )
		return false;
	else
		return true;
}

