#!/usr/bin/python
#
# Sakreble! - open word game.
# Copyright (C) 2004 Marcin Gajda
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#  
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

# Simple script to work with Sakreble! word dictionaries.

import sys, getopt, locale

class SakrebleDictionary:
	"""Dictionary class"""

	def __init__( self ):
		"""Initializes dictionary with empty dictionary.
		
		_wordlist is dictionary of lists, where keys are
		word lengths."""

		self._wordlist = {}
		self._withBigLetters = False
		self._withProgressBar = True
	
	def fromFile( self , fileName ):
		"""Adds words from file."""

		for word in file( fileName ):
			word = word.strip()

			if len( word ) > 0:
				if len( word ) not in self._wordlist.keys():
					self._wordlist[ len( word ) ] = []
		
				self._wordlist[ len( word ) ].append( word )
	
	def dump( self , fileName ):
		"""Dumps dictionary into file.
		
		All logic is here. Words are sorted and duplicates
		removed."""

		legths = self._wordlist.keys()
		legths.sort()

		f = open( fileName , "w" )
		prev = None

		if self._withProgressBar:
			counter = 10000L
			word_count = 0L
		
		for i in legths:
			self._wordlist[i].sort()

			for word in self._wordlist[i]:
				if self._withProgressBar:
					counter = counter - 1
					word_count = word_count + 1

					if counter == 0:
						print word_count + "..."

						counter = 10000L

				if prev == word:
					continue

				if not self._withBigLetters and word[0].isupper():
					continue
				
				f.write( word + "\n" )

				prev = word


	def setWithBigLetters( self , value ):
		"""Sets _withBigLetters."""

		self._withBigLetters = value
	
	def setWithProgressBar( self , value ):
		"""Sets _withProgressBar."""

		self._withProgressBar = value


def show_usage():
	print """Usage:

$ sakreble-dictionary [options] infile outfile

where options may be:

--with-big-letters - don't remove capitalized words
--for-locale=locale - use specified locales for capitalization determining
--without-progress-bar - don't show progress bar"""


def main():
	"""Main function."""
	
	# Parse options
	try:
		opts, args = getopt.getopt( sys.argv[1:] , "" , [ "with-big-letters" , "for-locale=" , "without-progress-bar" ] )
	except getopt.GetoptError:
		show_usage()

		sys.exit(2)

	# We need exactly two arguments
	if len( args ) != 2:
		show_usage()

		sys.exit(2)



	slownik = SakrebleDictionary()

	for option, argument in opts:
		if option == "--with-big-letters":
			slownik.setWithBigLetters( true )
			continue
		
		if option == "--for-locale":
			locale.setlocale( locale.LC_ALL , argument )
			continue

		if option == "--without-progress-bar":
			slownik.setWithProgressBar( False )
			continue


	slownik.fromFile( args[0] )
	slownik.dump( args[1] )

	sys.exit(0)


if __name__ == "__main__":
	main()

